unit Protokoll;

{#######################################################################################
Delphi UpdateLoader

Sendet ber RS232 Updates an den Bootloader "FastBoot" Version 2.1 von Peter Dannegger

Copyright (C) 2009 Leo-Andres Hofmann
Kontakt: leo.andres@gmx.de   |   webmaster@leo-andres.de

Die kommerzielle Nutzung, auch in Teilen, ist untersagt.

#######################################################################################}

interface

uses SysUtils, Windows;

const
  //Bootloader-Protokoll
  Bootloader_COMMAND: string = '$A5'; // Command sequence start
  Bootloader_ESCAPE: string  = '$A5';
  Bootloader_PASSWORDAPPEND: string = '$FF'; //added to the password

  Bootloader_CONNECT: string = '$A6';   // connection established
  Bootloader_BADCOMMAND: string = '$A7';// command not supported
  Bootloader_ANSWER: string = '$A8';    // followed by length byte
  Bootloader_CONTINUE: string = '$A9';
  Bootloader_SUCCESS: string = '$AA';
  Bootloader_FAIL: string = '$AB';

  Bootloader_ESC_SHIFT: string = '$80'; // offset escape char
  Bootloader_PROGEND: string   = '$80'; //finish program/verify

  Bootloader_REVISION: string  = '$0';   // get bootloader revision
  Bootloader_BUFFSIZE: string  = '$1';   // get buffer size
  Bootloader_SIGNATURE: string = '$2';   // get target signature
  Bootloader_USERFLASH: string = '$3';   // get user flash size
  Bootloader_PROGRAM: string   = '$4';   // program flash
  Bootloader_START: string     = '$5';   // start application
  Bootloader_CHECK_CRC: string = '$6';   // CRC o.k.
  Bootloader_VERIFY: string    = '$7';   // Verify

  //Bauteildefinitionen
  DeviceSignatures: array[0..20] of string = (
    '1E9007:ATtiny13',
    '1E910A:ATtiny2313',
    '1E9205:ATmega48',
    '1E9206:ATtiny45',
    '1E9207:ATtiny44',
    '1E9208:ATtiny461',
    '1E9306:ATmega8515',
    '1E9307:ATmega8',
    '1E9308:ATmega8535',
    '1E930A:ATmega88',
    '1E930B:ATtiny85',
    '1E930C:ATtiny84',
    '1E930D:ATtiny861',
    '1E930F:ATmega88P',
    '1E9403:ATmega16',
    '1E9404:ATmega162',
    '1E9406:ATmega168',
    '1E9501:ATmega323',
    '1E9502:ATmega32',
    '1E9609:ATmega644',
    '1E9802:ATmega2561');
  DefaultSignature: string = 'Unbekannt';

  //Programmeinstellungen
  ConnectTries: integer = 200;   //Verbindungsversuche

  ConnectTimeout: dword = 10;  //Timeout nach jedem Passwort-senden
  SendTimeout: dword    = 50;  //Timeout zum Buffer leeren
  GeneralTimeout: dword = 500; //Allgemeines warte-Timeout
  ReadValTimeout: dword = 3000;//Warten bis Info-Value bertragen
  VerifyTimeout: dword  = 3000;
  ProgramTimeout: dword = 5000;

  MaxFlashSize: longint = 262144; //Maximale Flashgre (256kb)
  ValidBootloaderVersion: string = '2.1'; //erlaubte Bootloader-Version

function HexToStr(HexStr: string): string;
function HexToChar(HexStr: string): char;
function BufferToHexStr(Buffer: byte): string;
function HexToVer(HexStr: string): string;
function HexToDevice(HexStr: string): string;
function HexToInt(HexStr: string): longint;
function ShiftEscape(HexStr: string): char;
function ShiftBufferEscape(Buffer: byte): char;
function BufferToChar(Buffer: byte): char;

implementation

function ByteToHex(Digit: byte): string;
const
  Digits: array[0..15] of char = '0123456789ABCDEF';
begin
  Result := '$' + (Digits[Digit shr 4] + Digits[Digit and $0F]);
end;

////////////////////////////////////////////////////////////////////////////////

function HexToStr(HexStr: string): string;
begin
  Result := Chr(StrToInt(HexStr));
end;

function HexToChar(HexStr: string): char;
begin
  Result := Chr(StrToInt(HexStr));
end;

function BufferToHexStr(Buffer: byte): string;
begin
  Result := ByteToHex(Buffer);
end;

function HexToVer(HexStr: string): string;
begin
  Result := IntToStr(StrToInt(copy(HexStr, 1, 2))) + '.' +
    IntToStr(StrToInt(copy(HexStr, 3, 2)));
end;

function HexToDevice(HexStr: string): string;
var
  I: integer;
begin
  Result := DefaultSignature;
  for I := low(DeviceSignatures) to high(DeviceSignatures) do
  begin
    if HexStr = copy(DeviceSignatures[I], 1, 6) then
    begin
      Result := copy(DeviceSignatures[I], 8, length(DeviceSignatures[I]));
    end;
  end;
end;

function HexToInt(HexStr: string): longint;
begin
  Result := StrToInt('$' + HexStr);
end;

function ShiftEscape(HexStr: string): char;
begin
  Result := Chr(StrToInt(HexStr) + StrToInt(Bootloader_ESC_SHIFT));
end;

function ShiftBufferEscape(Buffer: byte): char;
begin
  Result := ShiftEscape(BufferToHexStr(Buffer));
end;

function BufferToChar(Buffer: byte): char;
begin
  Result := Chr(StrToInt(ByteToHex(Buffer)));
end;

end.
